@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,11 +24,12 @@ from collections import defaultdict
 import datetime
 import sys
 import os
+import shutil
 import logging
 import logging.handlers
 import socket
-import platform
 
+from qt import QStandardPaths
 try:
     from sip import unwrapinstance
 except ImportError:
@@ -37,29 +38,16 @@ except ImportError:
         pass
 
 # pylint: disable=invalid-name
-if platform.python_version_tuple()[0] == '3':
-    # pylint: disable=redefined-builtin
-    unicode = str  # pylint: disable=W0622
-    bytes = bytes
-    long = int
-    isPython3 = True
-    interpreterName = 'python3'
-    xrange = range
+
+if os.name == 'nt':
+    # This is only needed for manual execution, and
+    # we expect python to be the python3 interpreter.
+    # The windows installer will use kajongg.exe and kajonggserver.exe
+    interpreterName = 'python'
 else:
-    # pylint: disable=redefined-builtin
-    unicode = unicode
-    bytes = str
-    long = long
-    isPython3 = False
-    if os.name == 'nt':
-        interpreterName = 'python'
-    else:
-        interpreterName = 'python2'
-    xrange = xrange
-
-LIGHTSOURCES = [u'NE', u'NW', u'SW', u'SE']
-ENGLISHDICT = {}
+    interpreterName = 'python3'
 
+LIGHTSOURCES = ['NE', 'NW', 'SW', 'SE']
 
 def isAlive(qobj):
     """is the underlying C++ object still valid?
@@ -76,7 +64,70 @@ def isAlive(qobj):
         return True
 
 
-class Debug(object):
+def appdataDir():
+    """
+    The per user directory with kajongg application information like the database.
+
+    @return: The directory path.
+    @rtype: C{str}.
+    """
+    serverDir = os.path.expanduser('~/.kajonggserver/')
+    if Internal.isServer:
+        # the server might or might not have KDE installed, so to be on
+        # the safe side we use our own .kajonggserver directory
+        # the following code moves an existing kajonggserver.db to .kajonggserver
+        # but only if .kajonggserver does not yet exist
+        kdehome = os.environ.get('KDEHOME', '~/.kde')
+        oldPath = os.path.expanduser(
+            kdehome +
+            '/share/apps/kajongg/kajonggserver.db')
+        if not os.path.exists(oldPath):
+            oldPath = os.path.expanduser(
+                '~/.kde' +'4/share/apps/kajongg/kajonggserver.db')
+        if os.path.exists(oldPath) and not os.path.exists(serverDir):
+            # upgrading an old kajonggserver installation
+            os.makedirs(serverDir)
+            shutil.move(oldPath, serverDir)
+        if not os.path.exists(serverDir):
+            try:
+                os.makedirs(serverDir)
+            except OSError:
+                pass
+        return serverDir
+    else:
+        if not os.path.exists(serverDir):
+            # the client wants to place the socket in serverDir
+            os.makedirs(serverDir)
+        result = QStandardPaths.writableLocation(QStandardPaths.AppDataLocation)
+        # this may end with kajongg.py or .pyw or whatever, so fix that:
+        result = os.path.join(os.path.dirname(result), 'kajongg')
+        if not os.path.exists(result):
+            os.makedirs(result)
+        return result
+
+
+def cacheDir():
+    """the cache directory for this user"""
+    result = os.path.join(appdataDir(), '.cache')
+    if not os.path.exists(result):
+        os.makedirs(result)
+    return result
+
+
+def socketName():
+    """client and server process use this socket to talk to each other"""
+    serverDir = os.path.expanduser('~/.kajonggserver')
+    if not os.path.exists(serverDir):
+        appdataDir()
+                   # allocate the directory and possibly move old databases
+                   # there
+    if Options.socket:
+        return Options.socket
+    else:
+        return os.path.normpath('{}/socket{}'.format(serverDir, Internal.defaultPort))
+
+
+class Debug:
 
     """holds flags for debugging output. At a later time we might
     want to add command line parameters for initialisation, and
@@ -86,7 +137,7 @@ class Debug(object):
     process = False
     time = False
     sql = False
-    animation = ''  # 'yeysywynfefsfwfn'
+    animation = ''  # 'yeysywynG87gfefsfwfn' for tiles and G#g for groups where # is the uid
     animationSpeed = False
     robotAI = False
     dangerousGame = False
@@ -110,12 +161,15 @@ class Debug(object):
     delayChow = False
     locate = False
     neutral = False  # only neutral comparable debug output
+    callers = '0'
     git = False
     ruleCache = False
     quit = False
     preferences = False
     graphics = False
     scoring = False
+    wallSize = '0'
+    i18n = False
 
     def __init__(self):
         raise Exception('Debug is not meant to be instantiated')
@@ -138,6 +192,7 @@ class Debug(object):
         allOptions = sorted(boolOptions + stringOptions)
         opt = '\n'.join(
             ', '.join(optYielder(allOptions)).split(' SEPARATOR, '))
+        # TODO: i18n for this string. First move i18n out of kde so we can import it here
         return """set debug options. Pass a comma separated list of options.
 Options are: {opt}.
 Options {stropt} take a string argument like {example}.
@@ -185,23 +240,25 @@ class FixedClass(type):
             type.__setattr__(cls, key, value)
 
 
-class StrMixin(object):
+class StrMixin:
 
     """
-    A mixin defining defaults for __str__ and __repr__,
-    using __unicode__.
+    A mixin defining a default for __repr__,
+    using __str__. If __str__ is not defined, this runs
+    into recursion. But I see no easy way without too much
+    runtime overhead to check for this beforehand.
     """
 
-    def __str__(self):
-        return nativeString(self.__unicode__())
-
     def __repr__(self):
-        return '{cls}({content})'.format(
-            cls=self.__class__.__name__,
-            content=self.__str__())
+        clsName = self.__class__.__name__
+        content = str(self)
+        if content.startswith(clsName):
+            return content
+        else:
+            return '{cls}({content})'.format(cls=clsName, content=content)
 
 
-class Options(object):
+class Options:
 
     """they are never saved in a config file. Some of them
     can be defined on the command line."""
@@ -227,7 +284,7 @@ class Options(object):
         raise Exception('Options is not meant to be instantiated')
 
 
-class SingleshotOptions(object):
+class SingleshotOptions:
 
     """Options which are cleared after having been used once"""
     table = False
@@ -235,7 +292,7 @@ class SingleshotOptions(object):
     game = None
 
 
-class __Internal(object):
+class __Internal:
 
     """
     Global things.
@@ -262,10 +319,7 @@ class __Internal(object):
     """
     # pylint: disable=too-many-instance-attributes
     Preferences = None
-    if isPython3:
-        defaultPort = 8300
-    else:
-        defaultPort = 8200
+    defaultPort = 8301
     logPrefix = 'C'
     isServer = False
     scaleScene = True
@@ -277,10 +331,13 @@ class __Internal(object):
     game = None
     autoPlay = False
     logger = None
+    kajonggrc = None
 
     def __init__(self):
         """init the loggers"""
-        logName = sys.argv[0].replace('.py', '') + '.log'
+        global Internal
+        Internal = self
+        logName = os.path.basename(sys.argv[0]).replace('.py', '').replace('.exe', '')  + '.log'
         self.logger = logging.getLogger(logName)
         if os.name == 'nt':
             haveDevLog = False
@@ -292,15 +349,17 @@ class __Internal(object):
                 haveDevLog = False
         if not haveDevLog:
             # pylint: disable=redefined-variable-type
+            logName = os.path.join(appdataDir(), logName)
+            print('Logging into {}'.format(logName))
             handler = logging.handlers.RotatingFileHandler(
-                'kajongg.log', maxBytes=100000000, backupCount=10)
+                logName, maxBytes=100000000, backupCount=10)
         self.logger.addHandler(handler)
         self.logger.addHandler(logging.StreamHandler(sys.stderr))
         self.logger.setLevel(logging.DEBUG)
         formatter = logging.Formatter("%(name)s: %(levelname)s %(message)s")
         handler.setFormatter(formatter)
 
-Internal = __Internal()
+__Internal()
 
 
 class IntDict(defaultdict, StrMixin):
@@ -354,7 +413,7 @@ class IntDict(defaultdict, StrMixin):
     def count(self, countFilter=None):
         """how many tiles defined by countFilter do we hold?
         countFilter is an iterator of element names. No countFilter: Take all
-        So count(['we','ws']) should return 8"""
+        So count(['we', 'ws']) should return 8"""
         return sum((defaultdict.get(self, x) or 0)
                    for x in countFilter or self)
 
@@ -362,7 +421,7 @@ class IntDict(defaultdict, StrMixin):
         """returns a list of all tiles defined by countFilter,
         each tile multiplied by its occurrence.
         countFilter is an iterator of element names. No countFilter: take all
-        So all(['we','fs']) should return ['we', 'we', 'we', 'we', 'fs']"""
+        So all(['we', 'fs']) should return ['we', 'we', 'we', 'we', 'fs']"""
         result = []
         for element in countFilter or self:
             result.extend([element] * self[element])
@@ -397,121 +456,18 @@ class IntDict(defaultdict, StrMixin):
         return ', '.join('{}:{}'.format(
             str(x), str(self[x])) for x in keys)
 
-    def __unicode__(self):
-        """sort the result for better log comparison"""
-        keys = sorted(self.keys())
-        return u', '.join('{}:{}'.format(
-            unicodeString(x), unicodeString(self[x])) for x in keys)
 
-
-class ZValues(object):
+class ZValues:
 
     """here we collect all zValues used in Kajongg"""
-    itemLevelFactor = 100000
-    boardLevelFactor = itemLevelFactor * 100
-    marker = boardLevelFactor * 100 + 1
-    moving = marker + 1
-    popup = moving + 1
+    itemZFactor = 100000
+    boardZFactor = itemZFactor * 100
+    markerZ = boardZFactor * 100 + 1
+    movingZ = markerZ + 1
+    popupZ = movingZ + 1
 
 
-def english(i18nstring):
-    """translate back from local language"""
-    return ENGLISHDICT.get(i18nstring, i18nstring)
-
-
-def unicodeString(s, encoding='utf-8'):
-    """
-    If s is not unicode, make it so.
-
-    @param s: The original string or None.
-    @type s: C{QString}, C{unicode}, C{str} or C{bytes}
-    @rtype: C{unicode} or None.
-    """
-    if s is None:
-        return s
-    if s.__class__.__name__ == 'QString':  # avoid import of QString
-        return unicode(s)
-    elif isinstance(s, unicode):
-        return s
-    elif hasattr(s, 'decode'):
-        return s.decode(encoding)
-    else:
-        return repr(s)
-
-
-def isStringType(s):
-    """Returns True for QString, QByteArray, str, bytes, unicode."""
-    if s.__class__.__name__ in ('QString', 'QByteArray'):
-        return True
-    return isinstance(s, (bytes, unicode))
-
-
-def nativeString(s, encoding='utf-8'):
-    """
-    Code inspired by twisted.python.compat.
-
-    Convert C{QByteArray}, C{QString}, C{bytes} or C{unicode}
-    to the native C{str} type, using the given encoding if
-    conversion is necessary.
-
-    @param s: The original string or None.
-    @type s: C{QByteArray}, C{QString}, C{unicode}, C{str} or C{bytes}
-    @param encoding: The encoding for the given string, if it is
-                not of type C{unicode}. Default is utf-8.
-    @returns: The string.
-    @rtype: C{str}
-
-    @raise UnicodeError: The input string is not encodable/decodable.
-    @raise TypeError: The input is not of string type.
-    """
-    if s is None:
-        return s
-    if s.__class__.__name__ == 'QString':  # avoid import of QString
-        s = unicode(s)
-    if s.__class__.__name__ == 'QByteArray':  # avoid import of QByteArray
-        s = bytes(s)
-    if not isStringType(s):
-        return s
-    if isPython3:
-        if isinstance(s, bytes):
-            return s.decode(encoding)
-        else:
-            # Ensure we're limited to the given encoding subset:
-            s.encode(encoding)
-    else:
-        if isinstance(s, unicode):
-            return s.encode(encoding)
-        else:
-            # Ensure we're limited to the given encoding subset:
-            s.decode(encoding)
-    return s
-
-
-def nativeStringArgs(args, encoding='utf-8'):
-    """
-    Convert string elements of a tuple to the native C{str} type,
-    Those elements which are not of some string type are left alone.
-    For acceptable string types see L{common.nativeString}.
-
-    @param args: A sequence of L{nativeString}.
-    @param encoding: The encoding for the strings. Default is utf-8.
-    @returns: A tuple with the converted strings.
-    @rtype: C{tuple}
-    """
-    return tuple((nativeString(x, encoding)
-                  if isStringType(x) else x for x in args))
-
-
-def unicodeStringArgs(args, encoding='utf-8'):
-    """
-    Convert string elements of a tuple to C{unicode},
-    Those elements which are not of some string type are left alone.
-    For acceptable string types see L{common.nativeString}.
-
-    @param s: None or a string to convert to C{str} if necessary.
-    @param encoding: The encoding for the strings. Default is utf-8.
-    @returns: A tuple with the converted strings.
-    @rtype: C{tuple}
-    """
-    return tuple((unicodeString(x, encoding)
-                  if isStringType(x) else x for x in args))
+class Speeds:
+    """some fixed animation speeds"""
+    windMarker = 20
+    sideText = 60
